"""Indicator serializer"""

from eea.dexterity.indicators.interfaces import IIndicator
from plone import api
from plone.restapi.interfaces import IObjectPrimaryFieldTarget
from zope.interface import implementer, Interface
from zope.component import adapter


@adapter(IIndicator, Interface)
@implementer(IObjectPrimaryFieldTarget)
class IndicatorObjectPrimaryFieldTarget:
    """Indicator primary field target"""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        if api.content.get_state(self.context) != "marked_for_deletion":
            return None

        url = self.context.absolute_url()
        old_version = url.split("-")[-1]
        try:
            int(old_version)
        except ValueError:
            return None

        return url.rstrip(old_version).rstrip("-")



"""
Patch JSON Summary Serializer (Copied from plone.restapi 8.x)

To be removed when switched to plone.restapi >= 8.x
"""

from plone.restapi.serializer.summary import DefaultJSONSummarySerializer
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.deserializer import json_body
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from eea.dexterity.indicators.interfaces import IEeaDexterityIndicatorsLayer


@implementer(ISerializeToJsonSummary)
@adapter(Interface, IEeaDexterityIndicatorsLayer)
class BackportJSONSummarySerializer(DefaultJSONSummarySerializer):
    """Backport ISerializeToJsonSummary adapter."""

    def metadata_fields(self):
        """Fields"""
        if not self.request.form:
            # maybe its a POST request
            query = json_body(self.request)
            self.request.form["metadata_fields"] = query.get("metadata_fields", [])
        return super().metadata_fields()



"""Upgrade step to upgrade indicators to the new embed block"""

from Products.CMFCore.utils import getToolByName
from Products.ZCatalog.ProgressHandler import ZLogHandler
from plone.restapi.blocks import visit_blocks


def to_110(context):
    """Upgrade all dataFigure blocks to embed_content"""
    ctool = getToolByName(context, "portal_catalog")
    brains = ctool.unrestrictedSearchResults(portal_type="ims_indicator")
    pghandler = ZLogHandler(100)
    pghandler.init("Convert dataFigure to embed_content blocks", len(brains))
    for idx, brain in enumerate(brains):
        pghandler.report(idx)
        doc = brain.getObject()
        for block in visit_blocks(doc, doc.blocks):
            block_type = block.get("@type", "")
            if block_type == "dataFigure":
                block["old_type"] = block_type
                block["@type"] = "embed_content"
                block["with_metadata_section"] = True
                block["svg_as_img"] = True
                block["with_notes"] = False
                href = block.get("href", "")
                if href and "/resolveuid/" in href:
                    block["old_url"] = block.get("url", "")
                    block["url"] = href
                doc.reindexObject()
    pghandler.finish()



"""Upgrade step to add new dividers to all indicators"""

from Products.CMFCore.utils import getToolByName
from Products.ZCatalog.ProgressHandler import ZLogHandler


def add_dividers_to_indicators(context):
    """Add new dividers to all indicators.

    Adds 43df8fab-b278-4b0e-a62c-ce6b8e0a881d and
    43df8fab-b278-4b0e-a62c-ce6b8e0a881e to all indicators.
    """
    ctool = getToolByName(context, "portal_catalog")
    brains = ctool.unrestrictedSearchResults(portal_type="ims_indicator")
    pghandler = ZLogHandler(100)
    pghandler.init("Add new dividers to all indicators", len(brains))

    # Define the new divider blocks
    new_dividers = {
        "43df8fab-b278-4b0e-a62c-ce6b8e0a881d": {
            "@type": "dividerBlock",
            "section": False,
            "short": True,
            "disableNewBlocks": True,
            "fixed": True,
            "hidden": True,
            "readOnly": True,
            "required": True,
            "styles": {},
            "spacing": "m",
            "fitted": False,
        },
        "43df8fab-b278-4b0e-a62c-ce6b8e0a881e": {
            "@type": "dividerBlock",
            "section": False,
            "short": True,
            "disableNewBlocks": True,
            "fixed": True,
            "hidden": True,
            "readOnly": True,
            "required": True,
            "spacing": "m",
            "fitted": False,
        },
    }

    def find_group_by_id_or_title(blocks, group_id, group_title):
        """Find group block by ID first, then by title"""
        # First try to find by ID
        if group_id in blocks:
            return blocks[group_id]

        # If not found by ID, search by title
        for block_data in blocks.values():
            if isinstance(block_data, dict) and block_data.get("title") == group_title:
                return block_data

        return None

    def add_divider_to_group(group, divider_id, embed_id, new_dividers):
        """Add divider to a group if it doesn't exist"""
        if not group or "data" not in group:
            return False

        group_data = group["data"]
        if "blocks" not in group_data:
            return False

        blocks = group_data["blocks"]

        # Add divider if it doesn't exist
        if divider_id in blocks:
            return False

        blocks[divider_id] = new_dividers[divider_id]

        # Add to layout if present
        if "blocks_layout" in group_data and "items" in group_data["blocks_layout"]:
            layout_items = group_data["blocks_layout"]["items"]
            if divider_id not in layout_items:
                # Insert after the embed_content block
                if embed_id in layout_items:
                    embed_index = layout_items.index(embed_id)
                    layout_items.insert(embed_index + 1, divider_id)
                else:
                    layout_items.append(divider_id)

        return True

    for idx, brain in enumerate(brains):
        pghandler.report(idx)
        doc = brain.getObject()

        # Check if the document has blocks
        if not hasattr(doc, "blocks") or not doc.blocks:
            continue

        blocks_modified = False

        # Find and update Aggregate level assessment group
        aggregate_id = "1bc4379d-cddb-4120-84ad-5ab025533b12"
        aggregate_title = "Aggregate level assessment"
        aggregate_group = find_group_by_id_or_title(
            doc.blocks, aggregate_id, aggregate_title
        )

        divider_id1 = "43df8fab-b278-4b0e-a62c-ce6b8e0a881d"
        embed_id1 = "b0279dde-1ceb-4137-a7f1-5ab7b46a782c"
        if add_divider_to_group(aggregate_group, divider_id1, embed_id1, new_dividers):
            blocks_modified = True

        # Find and update Disaggregate level assessment group
        disaggregate_id = "d060487d-88fc-4f7b-8ea4-003f14e0fb0c"
        disaggregate_title = "Disaggregate level assessment"
        disaggregate_group = find_group_by_id_or_title(
            doc.blocks, disaggregate_id, disaggregate_title
        )

        divider_id2 = "43df8fab-b278-4b0e-a62c-ce6b8e0a881e"
        embed_id2 = "02ba4a04-fcfe-4968-806f-1dac3119cfef"
        if add_divider_to_group(
            disaggregate_group, divider_id2, embed_id2, new_dividers
        ):
            blocks_modified = True

        if blocks_modified:
            doc.reindexObject()

    pghandler.finish()



"""eea.dexterity.indicators Installer"""

import os
from os.path import join
from setuptools import setup, find_packages

NAME = "eea.dexterity.indicators"
PATH = NAME.split(".") + ["version.txt"]
with open(join(*PATH), "r", encoding="utf-8") as version_file:
    VERSION = version_file.read().strip()

with open("README.rst", "r", encoding="utf-8") as readme_file:
    with open(
        os.path.join("docs", "HISTORY.txt"), "r", encoding="utf-8"
    ) as history_file:
        LONG_DESCRIPTION = readme_file.read() + "\n" + history_file.read()

setup(
    name=NAME,
    version=VERSION,
    description="Dexterity Indicators",
    long_description_content_type="text/x-rst",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="EEA Add-ons Plone Zope",
    author="European Environment Agency: IDM2 A-Team",
    author_email="eea-edw-a-team-alerts@googlegroups.com",
    url="https://github.com/eea/eea.dexterity.indicators",
    license="GPL version 2",
    packages=find_packages(exclude=["ez_setup"]),
    namespace_packages=["eea", "eea.dexterity"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        # -*- Extra requirements: -*-
        "plone.app.dexterity",
        "eea.api.layout",
        "eea.schema.slate",
        "eea.dexterity.themes",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



"""Upgrade to 9.9"""

from Products.CMFCore.utils import getToolByName
from Products.ZCatalog.ProgressHandler import ZLogHandler


def to_99(context):
    """Upgrade to 9.9"""
    ctool = getToolByName(context, "portal_catalog")
    brains = ctool.unrestrictedSearchResults(portal_type="ims_indicator")
    pghandler = ZLogHandler(100)
    pghandler.init("Reindex IMS indicator SearchableText", len(brains))
    for idx, brain in enumerate(brains):
        pghandler.report(idx)
        indicator = brain.getObject()
        indicator.reindexObject(idxs=["SearchableText"])
    pghandler.finish()



"""Copy action for content rules."""

from urllib.parse import urlparse
from Acquisition import aq_base
from OFS.event import ObjectClonedEvent
from OFS.SimpleItem import SimpleItem
import OFS.subscribers
from plone.restapi.blocks import visit_blocks
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.vocabularies.catalog import CatalogSource
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from plone.restapi.serializer.utils import uid_to_url
from plone.restapi.deserializer.utils import path2uid
from plone.uuid.interfaces import IUUID

try:
    from plone.base.utils import pretty_title_or_id
except ImportError:
    # BBB Plone 5
    from Products.CMFPlone.utils import pretty_title_or_id
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope import schema
from zope.component import adapter
from zope.event import notify
from zope.interface import implementer, Interface
from zope.lifecycleevent import ObjectCopiedEvent, modified


def getLink(path):
    """
    Get link
    """

    URL = urlparse(path)

    if URL.netloc.startswith("localhost") and URL.scheme:
        return path.replace(URL.scheme + "://" + URL.netloc, "")
    return path


def draftExistsFor(originalObj, new_id=None):
    """
    Check if an indicator has a draft already created.
    """
    if new_id and not new_id.endswith(".1"):
        return True
    if getattr(originalObj, "copied_to", None):
        return True
    return False


class ICopyAction(Interface):
    """Interface for the configurable aspects of a move action.

    This is also used to create add and edit forms, below.
    """

    target_folder = schema.Choice(
        title=_("Target folder"),
        description=_("As a path relative to the portal root."),
        required=True,
        source=CatalogSource(is_folderish=True),
    )

    change_note = schema.TextLine(
        title=_("Change note"),
        description=_("Optional change note to be used when creating new version."),
        required=False,
    )


@implementer(ICopyAction, IRuleElementData)
class CopyAction(SimpleItem):
    """The actual persistent implementation of the action element."""

    target_folder = ""
    change_note = ""
    element = "eea.dexterity.indicators.Copy"

    @property
    def summary(self):
        """A summary of the element's configuration."""
        return _("Copy to folder ${folder}.", mapping=dict(folder=self.target_folder))


@adapter(Interface, ICopyAction, Interface)
@implementer(IExecutable)
class CopyActionExecutor:
    """The executor for this action."""

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_url = getToolByName(self.context, "portal_url", None)
        if portal_url is None:
            return False

        obj = self.event.object
        previous_obj_path = obj.absolute_url_path()

        path = self.element.target_folder
        change_note = self.element.change_note
        if len(path) > 1 and path[0] == "/":
            path = path[1:]
        target = portal_url.getPortalObject().unrestrictedTraverse(
            str(path),
            None,
        )

        if target is None:
            self.error(
                obj,
                _("Target folder ${target} does not exist.", mapping={"target": path}),
            )
            return False

        old_id = obj.getId()
        new_id = self.generate_id(target, old_id)
        if draftExistsFor(obj, new_id):
            # Version already exists, redirect to it - refs #279130
            return True

        try:
            obj._notifyOfCopyTo(target, op=0)
        except ConflictError:
            raise
        except Exception as e:
            self.error(obj, str(e))
            return False

        orig_obj = obj
        obj = obj._getCopy(target)
        obj._setId(new_id)

        notify(ObjectCopiedEvent(obj, orig_obj))

        target._setObject(new_id, obj)
        obj = target._getOb(new_id)
        obj.wl_clearLocks()

        obj._postCopy(target, op=0)
        try:
            orig_obj.copied_to = IUUID(obj)
            obj.copied_from = IUUID(orig_obj)
        except Exception as e:
            self.error(obj, str(e))

        OFS.subscribers.compatibilityCall("manage_afterClone", obj, obj)

        notify(ObjectClonedEvent(obj))

        pr = getToolByName(obj, "portal_repository")
        pr.save(obj=obj, comment=change_note)
        # CHANGE URL OF FIGURES TO THE NEW DRAFT VERSION
        for block_data in visit_blocks(obj, obj.blocks):
            if block_data.get("@type") == "embed_content" and "url" in block_data:
                url = uid_to_url(block_data["url"])
                if previous_obj_path in url:
                    url = url.replace(previous_obj_path, previous_obj_path + ".1")
                    url = path2uid(context=self.context, link=getLink(url))
                    block_data["url"] = url
        modified(obj)
        return True

    def error(self, obj, error):
        """Report an error during the copy."""
        request = getattr(self.context, "REQUEST", None)
        if request is not None:
            title = pretty_title_or_id(obj, obj)
            message = _(
                "Unable to copy ${name} as part of content rule "
                "'copy' action: ${error}",
                mapping={"name": title, "error": error},
            )
            IStatusMessage(request).addStatusMessage(message, type="error")

    def generate_id(self, target, old_id):
        """Generate a new id for the copied object."""
        taken = getattr(aq_base(target), "has_key", lambda x: x in target.objectIds())

        if not taken(old_id):
            return old_id
        idx = 1
        while taken("{old_id}.{idx}".format(old_id=old_id, idx=idx)):
            idx += 1
        return "{old_id}.{idx}".format(old_id=old_id, idx=idx)


class CopyAddForm(ActionAddForm):
    """An add form for move-to-folder actions."""

    schema = ICopyAction
    label = _("Add Copy Action")
    description = _("A copy action can copy an object to a different folder.")
    Type = CopyAction


class CopyAddFormView(ContentRuleFormWrapper):
    """A wrapper for the add form."""

    form = CopyAddForm


class CopyEditForm(ActionEditForm):
    """An edit form for copy rule actions.

    z3c.form does all the magic here.
    """

    schema = ICopyAction
    label = _("Edit Copy Action")
    description = _("A copy action can copy an object to a different folder.")
    form_name = _("Configure element")


class CopyEditFormView(ContentRuleFormWrapper):
    """A wrapper for the edit form."""

    form = CopyEditForm
